"""
Django settings for jawanndenn project.

Generated by 'django-admin startproject' using Django 2.2.7.

For more information on this file, see
https://docs.djangoproject.com/en/2.2/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/2.2/ref/settings/
"""

import copy
import os

from django.utils.log import DEFAULT_LOGGING
from jawanndenn import DEFAULT_MAX_POLLS, DEFAULT_MAX_VOTES_PER_POLL

# Build paths inside the project like this: os.path.join(BASE_DIR, ...)
BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))


# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/2.2/howto/deployment/checklist/

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = os.environ['JAWANNDENN_SECRET_KEY']

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = os.environ.get('JAWANNDENN_DEBUG', 'False') == 'True'

ALLOWED_HOSTS = os.environ.get('JAWANNDENN_ALLOWED_HOSTS', ','.join([
    '127.0.0.1',
    '0.0.0.0',
    'localhost',
])).split(',')

_USE_POSTGRES = 'JAWANNDENN_SQLITE_FILE' not in os.environ


# Application definition

INSTALLED_APPS = [
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'jawanndenn',
]

if _USE_POSTGRES:
    INSTALLED_APPS += [
        'django_probes',  # management command "wait_for_database"
    ]

MIDDLEWARE = [
    'django.middleware.security.SecurityMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
]

ROOT_URLCONF = 'jawanndenn.urls'

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [
            os.path.join(BASE_DIR, 'jawanndenn', 'static'),
        ],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
            ],
        },
    },
]

WSGI_APPLICATION = 'jawanndenn.wsgi.application'


# Database
# https://docs.djangoproject.com/en/2.2/ref/settings/#databases

if _USE_POSTGRES:
    DATABASES = {
        'default': {
            'ENGINE': 'django.db.backends.postgresql',
            'NAME': os.environ['JAWANNDENN_POSTGRES_NAME'],
            'USER': os.environ['JAWANNDENN_POSTGRES_USER'],
            'PASSWORD': os.environ['JAWANNDENN_POSTGRES_PASSWORD'],
            'HOST': os.environ['JAWANNDENN_POSTGRES_HOST'],
            'PORT': os.environ['JAWANNDENN_POSTGRES_PORT'],
        }
    }
else:
    DATABASES = {
        'default': {
            'ENGINE': 'django.db.backends.sqlite3',
            'NAME': os.environ['JAWANNDENN_SQLITE_FILE'],
        }
    }


# Logging
# https://docs.djangoproject.com/en/2.2/ref/settings/#std:setting-LOGGING

LOGGING = copy.copy(DEFAULT_LOGGING)
LOGGING['handlers']['console']['filters'].remove('require_debug_true')
if not DEBUG:
    LOGGING['handlers']['console']['level'] = 'ERROR'


# Password validation
# https://docs.djangoproject.com/en/2.2/ref/settings/#auth-password-validators

AUTH_PASSWORD_VALIDATORS = [
    {
        'NAME': 'django.contrib.auth.password_validation.UserAttributeSimilarityValidator',  # noqa: E501
    },
    {
        'NAME': 'django.contrib.auth.password_validation.MinimumLengthValidator',  # noqa: E501
    },
    {
        'NAME': 'django.contrib.auth.password_validation.CommonPasswordValidator',  # noqa: E501
    },
    {
        'NAME': 'django.contrib.auth.password_validation.NumericPasswordValidator',  # noqa: E501
    },
]


# Internationalization
# https://docs.djangoproject.com/en/2.2/topics/i18n/

LANGUAGE_CODE = 'en-us'

TIME_ZONE = 'UTC'

USE_I18N = True

USE_L10N = True

USE_TZ = True


# Settings specific to jawanndenn

JAWANNDENN_URL_PREFIX = os.environ.get('JAWANNDENN_URL_PREFIX', '').strip('/')

if JAWANNDENN_URL_PREFIX != '':
    JAWANNDENN_URL_PREFIX = '/' + JAWANNDENN_URL_PREFIX

JAWANNDENN_MAX_POLLS = int(os.environ.get(
    'JAWANNDENN_MAX_POLLS', str(DEFAULT_MAX_POLLS)))
JAWANNDENN_MAX_VOTES_PER_POLL = int(os.environ.get(
    'JAWANNDENN_MAX_VOTES_PER_POLL', str(DEFAULT_MAX_VOTES_PER_POLL)))


# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/2.2/howto/static-files/

STATIC_URL = JAWANNDENN_URL_PREFIX + '/static/'

STATICFILES_DIRS = [
    os.path.join(BASE_DIR, 'jawanndenn', 'static'),
]



# Copyright (C) 2019 Sebastian Pipping <sebastian@pipping.org>
# Licensed under GNU Affero GPL v3 or later

import re

from django.conf import settings
from django.contrib import admin
from django.urls import include, path, re_path
from django.views.static import serve

from .views import (index_get_view, poll_data_get_view, poll_get_view,
                    poll_post_view, vote_post_view)


def _staticfiles_urlpatterns():
    '''
    Fork of django.contrib.staticfiles.urls.staticfiles_urlpatterns
    that supports DEBUG=False
    '''
    return [
        re_path(r'^%s(?P<path>.*)$' % re.escape(
            settings.STATIC_URL.lstrip('/')),
                serve, kwargs={
                'document_root': settings.STATICFILES_DIRS[0],
                'show_indexes': settings.DEBUG,
            }),
    ]


_app_urlpatterns = [
    path('', index_get_view, name='frontpage'),
    path('create', poll_post_view, name='poll-creation'),
    path('data/<poll_id>', poll_data_get_view, name='poll-data'),
    path('poll/<poll_id>', poll_get_view, name='poll-detail'),
    path('vote/<poll_id>', vote_post_view, name='vote'),

    path('admin/', admin.site.urls),
]

if settings.JAWANNDENN_URL_PREFIX:
    from django.views.generic.base import RedirectView
    urlpatterns = [
        path('', RedirectView.as_view(
            url=settings.JAWANNDENN_URL_PREFIX + '/')),
        path(settings.JAWANNDENN_URL_PREFIX.strip('/') + '/',
             include(_app_urlpatterns)),
    ]
else:
    urlpatterns = _app_urlpatterns

urlpatterns += _staticfiles_urlpatterns()



# Copyright (C) 2019 Sebastian Pipping <sebastian@pipping.org>
# Licensed under GNU Affero GPL v3 or later

import hashlib
import os

from django.db import models
from django.urls import reverse
from django_extensions.db.models import TimeStampedModel


def _get_random_sha256():
    return hashlib.sha256(os.urandom(256 // 8)).hexdigest()


class Poll(TimeStampedModel):
    slug = models.CharField(max_length=64, default=_get_random_sha256,
                            unique=True)
    title = models.CharField(max_length=255)
    equal_width = models.BooleanField(default=False)

    def get_absolute_url(self):
        return reverse('poll-detail', args=[self.slug])


class PollOption(models.Model):
    poll = models.ForeignKey(Poll, on_delete=models.CASCADE,
                             related_name='options')
    position = models.PositiveSmallIntegerField()  # starting at 0
    name = models.CharField(max_length=255)

    class Meta:
        unique_together = ('poll', 'position')


class Ballot(TimeStampedModel):
    poll = models.ForeignKey(Poll, on_delete=models.CASCADE,
                             related_name='ballots')
    voter_name = models.CharField(max_length=255)


class Vote(models.Model):
    ballot = models.ForeignKey(Ballot, related_name='votes',
                               on_delete=models.CASCADE)
    option = models.ForeignKey(PollOption, on_delete=models.CASCADE)
    yes = models.BooleanField()

    class Meta:
        unique_together = ('ballot', 'option')



# Copyright (C) 2016 Sebastian Pipping <sebastian@pipping.org>
# Licensed under GNU Affero GPL v3 or later

import argparse
import logging
import os
import signal
import string
import subprocess
import sys
from unittest.mock import patch

from jawanndenn import DEFAULT_MAX_POLLS, DEFAULT_MAX_VOTES_PER_POLL

_log = logging.getLogger(__name__)


def _require_hash_randomization():
    if not sys.flags.hash_randomization:
        _log.info('Hash randomization found to be disabled.')
        if os.environ.get('PYTHONHASHSEED') == 'random':
            _log.error('Unexpected re-execution loop detected, shutting down.')
            sys.exit(1)

        _log.info('Re-executing with hash randomization enabled...')
        env = os.environ.copy()
        env['PYTHONHASHSEED'] = 'random'
        argv = [sys.executable] + sys.argv
        os.execve(argv[0], argv, env)


def _generate_random_printable_django_secret_key():
    alphabet = ''.join(b for b in string.printable
                       if b not in string.whitespace and b not in '\'\\')
    # reduce number of retries while not introducing modulo bias
    alphabet = ''.join(alphabet for _ in range(256 // len(alphabet)))
    chars = []
    while len(chars) < 50:
        index = ord(os.urandom(1))
        if index >= len(alphabet):  # detect and avoid modulo bias
            continue
        chars.append(alphabet[index])
    return ''.join(chars)


def _process_django_secret_key_file(filename):
    secret_key_encoding = 'utf-8'
    try:
        with open(filename, 'r', encoding=secret_key_encoding) as f:
            secret_key = f.read()
    except FileNotFoundError:
        secret_key = _generate_random_printable_django_secret_key()
        with open(filename, 'w', encoding=secret_key_encoding) as f:
            f.write(secret_key)
    return secret_key


def _inner_main():
    parser = argparse.ArgumentParser(prog='jawanndenn')
    parser.add_argument('--debug', action='store_true', default=False,
                        help='Enable debug mode (default: disabled)')
    parser.add_argument('--host', default='127.0.0.1', metavar='HOST',
                        help='Hostname or IP address to listen at'
                             ' (default: %(default)s)')
    parser.add_argument('--port', default=8080, type=int, metavar='PORT',
                        help='Port to listen at (default: %(default)s)')
    parser.add_argument('--url-prefix', default='', metavar='PATH',
                        help='Path to prepend to URLs'
                             ' (default: "%(default)s")')
    parser.add_argument('--database-sqlite3', default='~/jawanndenn.sqlite3',
                        metavar='FILE',
                        help='File to write the database to'
                             ' (default: %(default)s)')
    parser.add_argument('--django-secret-key-file',
                        default='~/jawanndenn.secret_key', metavar='FILE',
                        help='File to use for Django secret key data'
                             ' (default: %(default)s)')

    limits = parser.add_argument_group('limit configuration')
    limits.add_argument('--max-polls', type=int, metavar='COUNT',
                        default=DEFAULT_MAX_POLLS,
                        help='Maximum number of polls total'
                             ' (default: %(default)s)')
    limits.add_argument('--max-votes-per-poll', type=int, metavar='COUNT',
                        default=DEFAULT_MAX_VOTES_PER_POLL,
                        help='Maximum number of votes per poll'
                             ' (default: %(default)s)')

    export_args = parser.add_argument_group('data import/export arguments')
    export_args.add_argument('--dumpdata', action='store_true',
                             help='Dump a JSON export of the database to '
                                  'standard output, then quit.')
    export_args.add_argument('--loaddata', metavar='FILE.json',
                             help='Load a JSON export of the database from '
                                  'FILE.json, then quit.')

    options = parser.parse_args()

    logging.basicConfig(level=logging.DEBUG if options.debug else logging.INFO)

    if not options.dumpdata and not options.loaddata:
        _require_hash_randomization()

    secret_key = _process_django_secret_key_file(
        os.path.expanduser(options.django_secret_key_file))

    # NOTE: These are read by the the Django settings module
    os.environ['JAWANNDENN_ALLOWED_HOSTS'] = ','.join([options.host,
                                                       '127.0.0.1',
                                                       '0.0.0.0',
                                                       'localhost'])
    os.environ['JAWANNDENN_DEBUG'] = str(options.debug)
    os.environ['JAWANNDENN_MAX_POLLS'] = str(options.max_polls)
    os.environ['JAWANNDENN_MAX_VOTES_PER_POLL'] = str(options
                                                      .max_votes_per_poll)
    os.environ['JAWANNDENN_SECRET_KEY'] = secret_key
    os.environ['JAWANNDENN_SQLITE_FILE'] = os.path.expanduser(
        options.database_sqlite3)
    os.environ['JAWANNDENN_URL_PREFIX'] = options.url_prefix

    os.environ.setdefault('DJANGO_SETTINGS_MODULE', 'jawanndenn.settings')

    # Heavy imports are down here to keep --help fast
    from django.core.management import execute_from_command_line

    with patch('sys.stdout', sys.stderr):
        execute_from_command_line(['./manage.py', 'migrate'])
        print()

    if options.dumpdata:
        execute_from_command_line(['./manage.py', 'dumpdata'])
    elif options.loaddata:
        print('Importing JSON dump'
              ' -- this may take a few seconds...', file=sys.stderr)
        execute_from_command_line(['./manage.py', 'loaddata',
                                   os.path.abspath(options.loaddata)])
    else:
        sys.exit(subprocess.call([
            'gunicorn',
            '--name=jawanndenn',
            f'--bind={options.host}:{options.port}',
            '--workers=1',  # due to use of sqlite3 storage
            '--access-logfile=-',
            '--logger-class=gunicorn_color.Logger',
            'jawanndenn.wsgi',
        ]))


def main():
    try:
        _inner_main()
    except KeyboardInterrupt:
        sys.exit(128 + signal.SIGINT)


if __name__ == '__main__':
    main()



# Generated by Django 2.2.7 on 2019-12-01 16:39

import django.db.models.deletion
import django_extensions.db.fields
import jawanndenn.models
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Poll',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', django_extensions.db.fields.CreationDateTimeField(auto_now_add=True, verbose_name='created')),
                ('modified', django_extensions.db.fields.ModificationDateTimeField(auto_now=True, verbose_name='modified')),
                ('slug', models.CharField(default=jawanndenn.models._get_random_sha256, max_length=64, unique=True)),
                ('title', models.CharField(max_length=255)),
                ('equal_width', models.BooleanField(default=False)),
            ],
            options={
                'ordering': ('-modified', '-created'),
                'get_latest_by': 'modified',
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='PollOption',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('position', models.PositiveSmallIntegerField()),
                ('name', models.CharField(max_length=255)),
                ('poll', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='options', to='jawanndenn.Poll')),
            ],
            options={
                'unique_together': {('poll', 'position')},
            },
        ),
        migrations.CreateModel(
            name='Ballot',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', django_extensions.db.fields.CreationDateTimeField(auto_now_add=True, verbose_name='created')),
                ('modified', django_extensions.db.fields.ModificationDateTimeField(auto_now=True, verbose_name='modified')),
                ('voter_name', models.CharField(max_length=255)),
                ('poll', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='ballots', to='jawanndenn.Poll')),
            ],
            options={
                'ordering': ('-modified', '-created'),
                'get_latest_by': 'modified',
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Vote',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('yes', models.BooleanField()),
                ('ballot', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='votes', to='jawanndenn.Ballot')),
                ('option', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='jawanndenn.PollOption')),
            ],
            options={
                'unique_together': {('ballot', 'option')},
            },
        ),
    ]



# Copyright (C) 2019 Sebastian Pipping <sebastian@pipping.org>
# Licensed under GNU Affero GPL v3 or later

import json
from functools import wraps
from json import JSONDecodeError

from django.conf import settings
from django.db import transaction
from django.http import (HttpResponseBadRequest, HttpResponseNotFound,
                         JsonResponse)
from django.shortcuts import redirect
from django.template.response import SimpleTemplateResponse
from django.views.decorators.http import require_GET, require_POST
from jawanndenn.markup import safe_html
from jawanndenn.models import Ballot, Poll, PollOption, Vote


def _except_poll_does_not_exist(wrappee):
    @wraps(wrappee)
    def wrapper(*args, **kwargs):
        try:
            return wrappee(*args, **kwargs)
        except Poll.DoesNotExist:
            return HttpResponseNotFound('No such poll')

    return wrapper


@require_GET
def index_get_view(request):
    context = {
        'url_prefix': settings.JAWANNDENN_URL_PREFIX,
    }

    response = SimpleTemplateResponse(template='html/setup.xhtml',
                                      context=context,
                                      content_type='application/xhtml+xml')
    response._request = request
    return response


@require_POST
def poll_post_view(request):
    try:
        config = json.loads(request.POST.get('config', '{}'))
    except JSONDecodeError:
        return HttpResponseBadRequest('Poll configuration is not '
                                      'well-formed JSON')
    poll_equal_width = bool(config.get('equal_width', False))
    poll_title = safe_html(config.get('title', ''))
    poll_option_names = map(safe_html, config.get('options', []))

    with transaction.atomic():
        if Poll.objects.count() >= settings.JAWANNDENN_MAX_POLLS:
            return HttpResponseBadRequest(
                f'Maximum number of {settings.JAWANNDENN_MAX_POLLS} polls '
                'reached, please contact the administrator.')

        poll = Poll.objects.create(title=poll_title,
                                   equal_width=poll_equal_width)
        for i, option_name in enumerate(poll_option_names):
            PollOption.objects.create(poll=poll, position=i, name=option_name)

    return redirect(poll)


@require_GET
@_except_poll_does_not_exist
def poll_data_get_view(request, poll_id):
    with transaction.atomic():
        poll = Poll.objects.get(slug=poll_id)
        poll_config = {
            'equal_width': poll.equal_width,
            'title': poll.title,
            'options': list(poll.options.order_by('position')
                            .values_list('name', flat=True)),
        }
        votes = [
            [ballot.voter_name, [vote.yes for vote
                                 in ballot.votes.order_by('option__position')]]
            for ballot
            in poll.ballots.order_by('created', 'id')
        ]

    data = {
        'config': poll_config,
        'votes': votes,
    }

    return JsonResponse(data)


@require_GET
@_except_poll_does_not_exist
def poll_get_view(request, poll_id):
    Poll.objects.get(slug=poll_id)

    context = {
        'url_prefix': settings.JAWANNDENN_URL_PREFIX,
    }

    response = SimpleTemplateResponse(template='html/poll.xhtml',
                                      context=context,
                                      content_type='application/xhtml+xml')
    response._request = request
    return response


@require_POST
@_except_poll_does_not_exist
def vote_post_view(request, poll_id):
    with transaction.atomic():
        poll = Poll.objects.get(slug=poll_id)

        if poll.ballots.count() >= settings.JAWANNDENN_MAX_VOTES_PER_POLL:
            return HttpResponseBadRequest(
                f'Maximum number of {settings.JAWANNDENN_MAX_VOTES_PER_POLL} '
                'votes reached for this poll'
                ', please contact the administrator.')

        voter_name = safe_html(request.POST.get('voterName'))
        votes = [
            request.POST.get(f'option{i}', 'off') == 'on'
            for i
            in range(poll.options.count())
        ]

        ballot = Ballot.objects.create(poll=poll, voter_name=voter_name)
        for option, vote in zip(poll.options.order_by('position'), votes):
            Vote.objects.create(ballot=ballot, option=option, yes=vote)

    return redirect(poll)



# Copyright (C) 2016 Sebastian Pipping <sebastian@pipping.org>
# Licensed under GNU Affero GPL v3 or later

from unittest import TestCase

from jawanndenn.markup import safe_html


class SafeHtmlTest(TestCase):
    def test_vanilla(self):
        self.assertEqual(safe_html(
                'text'),
                'text')

    def test_html(self):
        self.assertEqual(safe_html(
                '<b>&nbsp;</b>'),
                '&lt;b&gt;&amp;nbsp;&lt;/b&gt;')

    def test_bold(self):
        self.assertEqual(safe_html(
                '**text**'),
                '<strong>text</strong>')
        self.assertEqual(safe_html(
                '__text__'),
                '<strong>text</strong>')

    def test_italic(self):
        self.assertEqual(safe_html(
                '*text*'),
                '<em>text</em>')
        self.assertEqual(safe_html(
                '_text_'),
                '<em>text</em>')

    def test_monospace(self):
        self.assertEqual(safe_html(
                '`text`'),
                '<tt>text</tt>')

    def test_combined(self):
        self.assertEqual(safe_html(
                '`__*text*__`'),
                '<tt><strong><em>text</em></strong></tt>')
        self.assertEqual(safe_html(
                '`**_text_**`'),
                '<tt><strong><em>text</em></strong></tt>')
        self.assertEqual(safe_html(
                '_*text*_'),
                '<em><em>text</em></em>')

    def test_bad_nesting(self):
        self.assertEqual(safe_html(
                '*__text*__'),
                '<em><strong>text<em><strong></strong></em></strong></em>')

    def test_non_string(self):
        with self.assertRaises(ValueError):
            safe_html(123)



#! /usr/bin/env python3
# Copyright (C) 2016 Sebastian Pipping <sebastian@pipping.org>
# Licensed under GNU Affero GPL v3 or later

from setuptools import find_packages, setup

from jawanndenn.metadata import APP_NAME, VERSION_STR


def _read(filename):
    with open(filename, 'r') as f:
        return f.read()


if __name__ == '__main__':
    setup(
        name=APP_NAME,
        version=VERSION_STR,

        license='AGPLv3+',
        description='Libre alternative to Doodle',
        long_description=_read('README.rst'),

        author='Sebastian Pipping',
        author_email='sebastian@pipping.org',
        url='https://github.com/hartwork/jawanndenn',

        python_requires='>=3.6',
        install_requires=[
            'django>=2.2.7',
            'django-extensions>=2.2.5',
            'gunicorn>=20.0.4',
            'gunicorn-color>=0.1.0',
        ],

        packages=find_packages(),
        package_data={
            APP_NAME: [
                'static/css/*',
                'static/html/*',
                'static/js/*',
            ],
        },

        entry_points={
            'console_scripts': [
                f'{APP_NAME} = {APP_NAME}.__main__:main',
            ],
        },

        classifiers=[
            'Development Status :: 4 - Beta',
            'Framework :: Django',
            'Framework :: Django :: 2.2',
            'License :: OSI Approved :: GNU Affero General Public License v3 or later (AGPLv3+)',  # noqa: E501
            'License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)',  # noqa: E501
            'Programming Language :: JavaScript',
            'Programming Language :: Python',
            'Programming Language :: Python :: 3',
            'Programming Language :: Python :: 3.6',
            'Programming Language :: Python :: 3.7',
            'Programming Language :: Python :: 3.8',
            'Programming Language :: Python :: 3 :: Only',
            'Topic :: Internet :: WWW/HTTP :: WSGI :: Application',
            'Topic :: Office/Business :: Scheduling',
        ]
    )



# Copyright (C) 2019 Sebastian Pipping <sebastian@pipping.org>
# Licensed under GNU Affero GPL v3 or later

import json
from http import HTTPStatus

from django.test import TestCase
from django.urls import reverse
from django.utils import timezone
from jawanndenn.models import Ballot, Poll, PollOption, Vote


class IndexGetViewTest(TestCase):
    def test(self):
        url = reverse('frontpage')

        response = self.client.get(url)

        self.assertContains(response, 'Create a new poll')


class PollPostViewTest(TestCase):
    url = reverse('poll-creation')

    def test_malformed__config_not_json(self):
        data = {
            'config': 'not JSON',
        }
        before_creation = timezone.now()

        response = self.client.post(self.url, data)

        self.assertEqual(response.status_code, HTTPStatus.BAD_REQUEST)
        self.assertFalse(Poll.objects.filter(created__gte=before_creation)
                         .exists())

    def test_well_formed(self):
        poll_title = 'Some short title'
        poll_option_names = ['Option One', 'Option Two']
        poll_equal_width = True
        data = {
            'config': json.dumps({
                'equal_width': poll_equal_width,
                'title': poll_title,
                'options': poll_option_names,
            }),
        }
        before_creation = timezone.now()

        response = self.client.post(self.url, data)

        poll = Poll.objects.get(created__gte=before_creation)
        self.assertEqual(poll.equal_width, poll_equal_width)
        self.assertEqual(list(poll.options
                              .order_by('position')
                              .values_list('name', flat=True)),
                         poll_option_names)
        self.assertEqual(poll.title, poll_title)

        self.assertEqual(response.status_code, HTTPStatus.FOUND)
        self.assertEqual(response.url, poll.get_absolute_url())


class PollDataGetViewTest(TestCase):
    @classmethod
    def setUpTestData(cls):
        super().setUpTestData()
        cls.poll_option_names = ['Option One', 'Option Two']
        cls.poll = Poll.objects.create(title='Some short title',
                                       equal_width=True)
        cls.poll_options = [
            PollOption.objects.create(poll=cls.poll, position=position,
                                      name=option_name)
            for position, option_name
            in enumerate(cls.poll_option_names)
        ]
        cls.ballot = Ballot.objects.create(poll=cls.poll, voter_name='Maria')
        cls.votes = [
            Vote.objects.create(ballot=cls.ballot, option=option,
                                yes=option == cls.poll_options[0])
            for option
            in cls.poll_options
        ]

    def test_poll_exists(self):
        url = reverse('poll-data', args=[self.poll.slug])
        expected_data = {
            'config': {
                'equal_width': self.poll.equal_width,
                'options': self.poll_option_names,
                'title': self.poll.title,
            },
            'votes': [
                [self.ballot.voter_name, [v.yes for v in self.votes]],
            ],
        }

        response = self.client.get(url)

        actual_data = json.loads(response.content)
        self.assertEqual(actual_data, expected_data)
        self.assertEqual(response.status_code, HTTPStatus.OK)

    def test_poll_does_not_exist(self):
        url = reverse('poll-data', args=['whatever'])

        response = self.client.get(url)

        self.assertEqual(response.status_code, HTTPStatus.NOT_FOUND)


class PollGetViewTest(TestCase):
    @classmethod
    def setUpTestData(cls):
        super().setUpTestData()
        cls.poll = Poll.objects.create()

    def test_poll_exists(self):
        url = self.poll.get_absolute_url()

        response = self.client.get(url)

        self.assertContains(response, 'Vote!')

    def test_poll_does_not_exist(self):
        url = reverse('poll-detail', args=['no_such_poll'])

        response = self.client.get(url)

        self.assertEqual(response.status_code, HTTPStatus.NOT_FOUND)


class VotePostViewTest(TestCase):
    @classmethod
    def setUpTestData(cls):
        super().setUpTestData()
        cls.poll = Poll.objects.create()
        for position, option_name in enumerate(('Option One', 'Option Two')):
            PollOption.objects.create(poll=cls.poll, position=position,
                                      name=option_name)

    def test_poll_exists(self):
        url = reverse('vote', args=[self.poll.slug])
        voter_name = 'Maria'
        data = {
            'voterName': voter_name,
            'option0': 'on',
            'option1': 'off',
        }

        response = self.client.post(url, data)

        ballot = Ballot.objects.get(poll=self.poll)
        self.assertEqual(list(ballot.votes
                              .order_by('option__position')
                              .values_list('yes', flat=True)), [True, False])

        self.assertEqual(response.status_code, HTTPStatus.FOUND)
        self.assertEqual(response.url, self.poll.get_absolute_url())

    def test_poll_does_not_exist(self):
        url = reverse('vote', args=['no_such_poll'])
        data = {}

        response = self.client.post(url, data)

        self.assertEqual(response.status_code, HTTPStatus.NOT_FOUND)
